'''
Created on 16 Jul 2009

@author: charanpal
'''
from apgl.util import *
import numpy
import logging

class AbstractPredictor(object):
    """
    An abstract classifier for binary labelled data. 
    """
    def learnModel(self, X, y):
        """
        Learn a model for a set of examples given as the rows of the matrix X,
        with corresponding labels given in the rows of y. 
        """
        Util.abstract()
    
    def predict(self, X):
        """
        Make a prediction for a set of examples given as the rows of the matrix X.
        """
        Util.abstract()
    
    def evaluateCv(self, X, y, folds, errorMethod=Evaluator.binaryError):
        """
        Evaluate this classifier using cross validation with the specific number
        of folds on the given examplesList.

        Output: the mean and variation of the cross validation folds. 
        """
        Parameter.checkInt(folds, 2, X.shape[0])
        Parameter.checkInt(X.shape[0], 1, float('inf'))
        
        idx = Sampling.crossValidation(folds, y.shape[0])
        i = 0
        errs = numpy.zeros(len(idx))

        logging.info("Computing " + str(folds) + "-fold cross validation on " + str(X.shape[0]) + " examples")

        for idxtr, idxts in idx:
            logging.info("Iteration " + str(i))
            trainX, testX = X[idxtr], X[idxts]
            trainY, testY = y[idxtr], y[idxts]

            self.learnModel(trainX, trainY)
            predY = self.predict(testX)

            errs[i] = errorMethod(testY, predY)
            i += 1

        mean = numpy.mean(errs, 0)
        var = numpy.var(errs, 0)

        return (mean, var)